import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';

class DatetimeFieldWidget extends StatefulWidget implements IListItem {
  final bool isFirst;
  final bool isLast;
  final Text title;
  final bool withDate;
  final bool withTime;
  final bool onlyMonthAndYear;
  final bool withPicker;
  final bool withDeletion;

  final String emptyDateString;

  final FieldValidation validation;
  final void Function(DateTime value) saveValue;
  final DateTime Function() getValue;

  final bool isEditable;
  final bool isDisabled;

  final HighlightController highlightController;

  final FocusNode focusNode;
  final FocusNode nextFocusNode;

  const DatetimeFieldWidget({
    Key key,
    this.title,
    this.saveValue,
    @required this.getValue,
    this.withDate = true,
    this.withTime = true,
    this.validation,
    @required this.isEditable,
    this.emptyDateString = "-",
    this.isDisabled = false,
    this.onlyMonthAndYear = false,
    this.withPicker = true,
    this.withDeletion = true,
    this.highlightController,
    this.focusNode,
    this.nextFocusNode,
  })  : assert(title != null),
        assert(getValue != null),
        assert(!(isEditable ?? true) || saveValue != null),
        isLast = null,
        isFirst = null,
        super(key: key);

  const DatetimeFieldWidget._copy({
    Key key,
    this.isFirst,
    this.isLast,
    this.title,
    this.withDate,
    this.withTime,
    this.saveValue,
    this.getValue,
    this.isEditable,
    this.validation,
    this.emptyDateString,
    this.isDisabled,
    this.onlyMonthAndYear,
    this.withPicker,
    this.withDeletion,
    this.highlightController,
    this.focusNode,
    this.nextFocusNode,
  }) : super(key: key);

  @override
  _DatetimeFieldWidgetState createState() => _DatetimeFieldWidgetState();

  @override
  IListItem copyWithPosition({bool isFirst, bool isLast}) {
    return DatetimeFieldWidget._copy(
      key: this.key,
      title: this.title,
      getValue: this.getValue,
      saveValue: this.saveValue,
      withDate: this.withDate,
      withTime: this.withTime,
      isEditable: this.isEditable,
      validation: this.validation,
      emptyDateString: this.emptyDateString,
      onlyMonthAndYear: this.onlyMonthAndYear,
      withPicker: this.withPicker,
      withDeletion: this.withDeletion,
      isDisabled: this.isDisabled,
      highlightController: this.highlightController,
      focusNode: this.focusNode,
      nextFocusNode: this.nextFocusNode,
      isFirst: isFirst ?? this.isFirst,
      isLast: isLast ?? this.isLast,
    );
  }
}

class _DatetimeFieldWidgetState extends State<DatetimeFieldWidget> {
  String errorMessage;

  bool get isEditable => widget.isEditable && !widget.isDisabled;

  var focusNodeD0 = FocusNode();
  var focusNodeD1 = FocusNode();
  var focusNodeD2 = FocusNode();
  var focusNodeD3 = FocusNode();
  var focusNodeD4 = FocusNode();
  var focusNodeD5 = FocusNode();
  var controllerD0 = TextEditingController();
  var controllerD1 = TextEditingController();
  var controllerD2 = TextEditingController();
  bool fieldD1IsEmpty = false;
  bool fieldD2IsEmpty = false;
  var focusNodeT0 = FocusNode();
  var focusNodeT1 = FocusNode();
  var focusNodeT2 = FocusNode();
  var focusNodeT3 = FocusNode();
  var controllerT0 = TextEditingController();
  var controllerT1 = TextEditingController();
  bool fieldT1IsEmpty = false;

  bool validDate = true;

  DateTime lastShowedDate;
  bool lastIsEditable;

  @override
  void initState() {
    super.initState();

    widget.validation?.connectToField(
      onValidate: (errorMessage) {
        if (mounted) setState(() => this.errorMessage = errorMessage);
      },
    );

    if (widget.withDate && widget.focusNode != null)
      focusNodeD0 = widget.focusNode;
    else if (!widget.withDate && widget.withTime && widget.focusNode != null)
      focusNodeT0 = widget.focusNode;

    if (isEditable) setNewDateTime(widget.getValue(), init: true);

    focusNodeD0.addListener(() => selectAll(focusNodeD0, controllerD0, 2));
    focusNodeD1.addListener(() => selectAll(focusNodeD1, controllerD1, 2));
    focusNodeD2.addListener(() => selectAll(focusNodeD2, controllerD2, 4));
    focusNodeT0.addListener(() => selectAll(focusNodeT0, controllerT0, 2));
    focusNodeT1.addListener(() => selectAll(focusNodeT1, controllerT1, 2));

    if (!kIsWeb) {
      // replaces the RawKeyboardListeners
      controllerD0.addListener(() {
        if (controllerD0.text.length == 2) {
          if (!(controllerD0.selection.baseOffset == 0 &&
              controllerD0.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeD1);
        }
        parseDateTimeControllers();
      });

      controllerD1.addListener(() {
        if (controllerD1.text.length == 2) {
          if (!(controllerD1.selection.baseOffset == 0 &&
              controllerD1.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeD2);
        }
        if (controllerD1.text.length == 0) {
          fieldD1IsEmpty = true;
        } else {
          fieldD1IsEmpty = false;
        }
        parseDateTimeControllers();
      });

      controllerD2.addListener(() {
        if (controllerD2.text.length == 0) {
          fieldD2IsEmpty = true;
        } else {
          fieldD2IsEmpty = false;
        }
        parseDateTimeControllers();
      });

      controllerT0.addListener(() {
        if (controllerT0.text.length == 2) {
          if (!(controllerT0.selection.baseOffset == 0 &&
              controllerT0.selection.extentOffset == 2))
            FocusScope.of(context).requestFocus(focusNodeT1);
        }
        parseDateTimeControllers();
      });

      controllerT1.addListener(() {
        if (controllerT1.text.length == 0) {
          fieldT1IsEmpty = true;
        } else {
          fieldT1IsEmpty = false;
        }
        parseDateTimeControllers();
      });
    }
  }

  void selectAll(
      FocusNode focusNode, TextEditingController controller, int count) {
    if (focusNode.hasFocus && controller.text.length == count)
      controller.value = controller.value.copyWith(
        text: controller.text,
        selection:
            TextSelection(baseOffset: 0, extentOffset: controller.text.length),
        composing: TextRange.empty,
      );
  }

  void parseDateTimeControllers() {
    var minuteString = widget.withTime
        ? (controllerT1.text.length == 2
            ? controllerT1.text
            : ("0" + controllerT1.text))
        : "00";
    var hourString = widget.withTime
        ? (controllerT0.text.length == 2
            ? controllerT0.text
            : ("0" + controllerT0.text))
        : "00";
    var dayString = (widget.withDate && !widget.onlyMonthAndYear)
        ? (controllerD0.text.length == 2
            ? controllerD0.text
            : ("0" + controllerD0.text))
        : "01";
    var monthString = widget.withDate
        ? (controllerD1.text.length == 2
            ? controllerD1.text
            : ("0" + controllerD1.text))
        : "01";
    var yearString = widget.withDate
        ? (controllerD2.text.length == 4
            ? controllerD2.text
            : (controllerD2.text.length == 2 ? ("20" + controllerD2.text) : ""))
        : "2000";
    var s = "$dayString.$monthString.$yearString $minuteString:$hourString";
    try {
      var date = DateFormat("dd.MM.yyyy mm:HH").parse(s);
      if (DateFormat("dd.MM.yyyy mm:HH").format(date) == s) {
        widget.saveValue?.call(date);
        lastShowedDate = date;
        setState(() {
          validDate = true;
        });
      } else {
        setState(() {
          validDate = false;
        });
      }
    } on FormatException {
      setState(() {
        validDate = false;
      });
    }
  }

  void setNewDateTime(DateTime dateTime, {bool init = false}) {
    widget.saveValue?.call(dateTime);
    lastShowedDate = dateTime;

    controllerD0.text =
        dateTime == null ? "" : DateFormat("dd", "de_DE").format(dateTime);
    controllerD1.text =
        dateTime == null ? "" : DateFormat("MM", "de_DE").format(dateTime);
    controllerD2.text =
        dateTime == null ? "" : DateFormat("yyyy", "de_DE").format(dateTime);
    controllerT1.text =
        dateTime == null ? "" : DateFormat("mm", "de_DE").format(dateTime);
    controllerT0.text =
        dateTime == null ? "" : DateFormat("HH", "de_DE").format(dateTime);

    if (!init) setState(() {});
  }

  bool datesAreTheSame(DateTime d0, DateTime d1,
      {bool compareDate = true, bool compareTime = true}) {
    //print("d0: ${d0 == null ? 'null' : DateFormat("dd.MM.yyyy").format(d0)}");
    //print("d1: ${d1 == null ? 'null' : DateFormat("dd.MM.yyyy").format(d1)}");

    if (d0 == null || d1 == null) return true;
    if (compareDate) {
      if (d0.year != d1.year || d0.month != d1.month || d0.day != d1.day)
        return false;
    }
    if (compareTime) {
      if (d0.hour != d1.hour || d0.minute != d1.minute) return false;
    }
    return true;
  }

  @override
  Widget build(BuildContext context) {
    if ((lastIsEditable != isEditable) ||
        (isEditable &&
            !datesAreTheSame(lastShowedDate, widget.getValue(),
                compareDate: widget.withDate, compareTime: widget.withTime))) {
      setNewDateTime(widget.getValue(), init: true);
    }
    lastIsEditable = isEditable;

    return DefaultFieldWidget(
      isFirst: widget.isFirst,
      isLast: widget.isLast,
      separationLine: true,
      title: widget.title,
      highlightController: widget.highlightController,
      childPadding:
          isEditable ? EdgeInsets.only(left: 30, right: 30, bottom: 3) : null,
      child: Column(
        children: <Widget>[
          Row(
            children: <Widget>[
              _getDateWidget(context),
              SizedBox(
                width: (widget.withDate && widget.withTime) ? 15 : 0,
              ),
              _getTimeWidget(),
              (widget.withDeletion && isEditable)
                  ? IconButton(
                      icon: Icon(
                        Icons.highlight_off,
                        color: CleanAndroidDesignDefaultColors.defaultColor,
                      ),
                      tooltip:
                          "${widget.withDate ? 'Datum' : 'Uhrzeit'} entfernen",
                      onPressed: () {
                        FocusScope.of(context).unfocus();
                        setNewDateTime(null);
                      },
                    )
                  : Container(),
            ],
          ),
          errorMessage != null
              ? Align(
                  alignment: Alignment.centerLeft,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 6),
                    child: Text(
                      errorMessage,
                      style: TextStyles.textStyleSize(12).copyWith(
                          color: CleanAndroidDesignDefaultColors.error),
                    ),
                  ),
                )
              : Container(),
        ],
      ),
    );
  }

  Widget _getDateWidget(BuildContext context) {
    if (!widget.withDate) return Container();

    var date = widget.getValue();

    if (!isEditable) {
      var dateString = date == null
          ? widget.emptyDateString
          : DateFormat(widget.onlyMonthAndYear ? "MMM yyyy" : "EEE dd.MM.yyy",
                  "de_DE")
              .format(date);
      return AdaptiveText(
        dateString,
        style: TextStyles.textStyleSize(17).copyWith(
          color: widget.isDisabled
              ? Colors.grey[500]
              : CleanAndroidDesignDefaultColors.black,
        ),
      );
    }

    var inputBorder = (!widget.isLast || !validDate)
        ? UnderlineInputBorder(
            borderSide: BorderSide(
                color: validDate
                    ? CleanAndroidDesignDefaultColors.defaultColorLight
                    : CleanAndroidDesignDefaultColors.error.withOpacity(0.85),
                width: 1))
        : InputBorder.none;
    var focusedBorder = UnderlineInputBorder(
        borderSide: BorderSide(
      color: validDate
          ? CleanAndroidDesignDefaultColors.accent
          : CleanAndroidDesignDefaultColors.error,
      width: 2,
    ));
    var decoration = InputDecoration(
      contentPadding: EdgeInsets.only(top: 2, bottom: 5),
      isDense: true,
      enabledBorder: inputBorder,
      disabledBorder: inputBorder,
      focusedBorder: focusedBorder,
    );

    return Row(
      children: [
        widget.onlyMonthAndYear
            ? Container()
            : Container(
                width: 30,
                child: RawKeyboardListener(
                  focusNode: focusNodeD3,
                  onKey: kIsWeb
                      ? (value) {
                          if (controllerD0.text.length == 2) {
                            if (!(controllerD0.selection.baseOffset == 0 &&
                                controllerD0.selection.extentOffset == 2))
                              FocusScope.of(context).requestFocus(focusNodeD1);
                          }
                          parseDateTimeControllers();
                        }
                      : null,
                  child: TextField(
                    textAlign: TextAlign.center,
                    controller: controllerD0,
                    focusNode: focusNodeD0,
                    maxLength: 2,
                    cursorColor: CleanAndroidDesignDefaultColors.accentLight,
                    decoration: decoration.copyWith(hintText: "TT"),
                    buildCounter: (context,
                            {currentLength, isFocused, maxLength}) =>
                        null,
                    keyboardType: TextInputType.datetime,
                    textInputAction: TextInputAction.next,
                    onSubmitted: (value) {
                      FocusScope.of(context).requestFocus(focusNodeD1);
                    },
                  ),
                ),
              ),
        widget.onlyMonthAndYear ? Container() : Text("/"),
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeD4,
            onKey: kIsWeb
                ? (value) {
                    if (controllerD1.text.length == 2) {
                      if (!(controllerD1.selection.baseOffset == 0 &&
                          controllerD1.selection.extentOffset == 2))
                        FocusScope.of(context).requestFocus(focusNodeD2);
                    }
                    if (controllerD1.text.length == 0) {
                      if (fieldD1IsEmpty &&
                          !widget.onlyMonthAndYear &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerD0.text.length > 0)
                          controllerD0.text = controllerD0.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeD0));
                      }
                      fieldD1IsEmpty = true;
                    } else {
                      fieldD1IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerD1,
              focusNode: focusNodeD1,
              maxLength: 2,
              cursorColor: CleanAndroidDesignDefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "MM"),
              buildCounter: (context, {currentLength, isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: TextInputAction.next,
              onSubmitted: (value) {
                FocusScope.of(context).requestFocus(focusNodeD2);
              },
            ),
          ),
        ),
        Text("/"),
        Container(
          width: 50,
          child: RawKeyboardListener(
            focusNode: focusNodeD5,
            onKey: kIsWeb
                ? (value) {
                    if (controllerD2.text.length == 4) {
                      if (!(controllerD2.selection.baseOffset == 0 &&
                          controllerD2.selection.extentOffset ==
                              4)) if (widget.withTime)
                        FocusScope.of(context).requestFocus(focusNodeT0);
                    }

                    if (controllerD2.text.length == 0) {
                      if (fieldD2IsEmpty &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerD1.text.length > 0)
                          controllerD1.text = controllerD1.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeD1));
                      }
                      fieldD2IsEmpty = true;
                    } else {
                      fieldD2IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerD2,
              focusNode: focusNodeD2,
              maxLength: 4,
              cursorColor: CleanAndroidDesignDefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "JJJJ"),
              buildCounter: (context, {currentLength, isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: (widget.withTime || widget.nextFocusNode != null)
                  ? TextInputAction.next
                  : TextInputAction.done,
              onSubmitted: (value) {
                if (widget.withTime)
                  FocusScope.of(context).requestFocus(focusNodeT0);
                else if (widget.nextFocusNode != null)
                  FocusScope.of(context).requestFocus(widget.nextFocusNode);
              },
            ),
          ),
        ),
        widget.withPicker
            ? IconButton(
                icon: Icon(
                  Icons.event,
                  color: CleanAndroidDesignDefaultColors.defaultColor,
                ),
                tooltip: "Auswahl öffnen",
                onPressed: () {
                  FocusScope.of(context).unfocus();
                  showCustomDialog(
                      context,
                      DatetimePickerDialog(
                        datetime: date,
                        onlyMonthAndYear: widget.onlyMonthAndYear,
                        setDatetime: (date) => setNewDateTime(date),
                      ));
                },
              )
            : Container(),
      ],
    );
  }

  Widget _getTimeWidget() {
    if (!widget.withTime) return Container();

    var date = widget.getValue();

    if (!isEditable) {
      var dateString = date == null
          ? (isEditable ? "Uhrzeit" : (widget.withDate ? "" : "-"))
          : DateFormat("HH:mm").format(date);
      return AdaptiveText(
        dateString,
        style: TextStyles.textStyleSize(17).copyWith(
          color: widget.isDisabled
              ? Colors.grey[500]
              : CleanAndroidDesignDefaultColors.black,
        ),
      );
    }

    var inputBorder = !widget.isLast
        ? UnderlineInputBorder(
            borderSide: BorderSide(
                color: CleanAndroidDesignDefaultColors.defaultColorLight,
                width: 1))
        : InputBorder.none;
    var focusedBorder = UnderlineInputBorder(
        borderSide: BorderSide(
      color: validDate
          ? CleanAndroidDesignDefaultColors.accent
          : CleanAndroidDesignDefaultColors.error,
      width: 2,
    ));
    var decoration = InputDecoration(
      contentPadding: EdgeInsets.only(top: 2, bottom: 5),
      isDense: true,
      enabledBorder: inputBorder,
      disabledBorder: inputBorder,
      focusedBorder: focusedBorder,
    );

    return Row(
      children: [
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeT2,
            onKey: kIsWeb
                ? (value) {
                    if (controllerT0.text.length == 2) {
                      if (!(controllerT0.selection.baseOffset == 0 &&
                          controllerT0.selection.extentOffset == 2)) {
                        FocusScope.of(context).requestFocus(focusNodeT1);
                      }
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerT0,
              focusNode: focusNodeT0,
              maxLength: 2,
              cursorColor: CleanAndroidDesignDefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "hh"),
              buildCounter: (context, {currentLength, isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: TextInputAction.next,
              onSubmitted: (value) {
                FocusScope.of(context).requestFocus(focusNodeT1);
              },
            ),
          ),
        ),
        Text(":"),
        Container(
          width: 30,
          child: RawKeyboardListener(
            focusNode: focusNodeT3,
            onKey: kIsWeb
                ? (value) {
                    if (controllerT1.text.length == 0) {
                      if (fieldT1IsEmpty &&
                          value.physicalKey == PhysicalKeyboardKey.backspace) {
                        if (controllerT0.text.length > 0)
                          controllerT0.text = controllerT0.text.substring(0, 1);
                        Future.delayed(
                            Duration.zero,
                            () => FocusScope.of(context)
                                .requestFocus(focusNodeT0));
                      }
                      fieldT1IsEmpty = true;
                    } else {
                      fieldT1IsEmpty = false;
                    }
                    parseDateTimeControllers();
                  }
                : null,
            child: TextField(
              textAlign: TextAlign.center,
              controller: controllerT1,
              focusNode: focusNodeT1,
              maxLength: 2,
              cursorColor: CleanAndroidDesignDefaultColors.accentLight,
              decoration: decoration.copyWith(hintText: "mm"),
              buildCounter: (context, {currentLength, isFocused, maxLength}) =>
                  null,
              keyboardType: TextInputType.datetime,
              textInputAction: widget.nextFocusNode != null
                  ? TextInputAction.next
                  : TextInputAction.done,
              onSubmitted: (value) {
                if (widget.nextFocusNode != null)
                  FocusScope.of(context).requestFocus(widget.nextFocusNode);
              },
            ),
          ),
        ),
        widget.withPicker
            ? IconButton(
                icon: Icon(
                  Icons.schedule,
                  color: CleanAndroidDesignDefaultColors.defaultColor,
                ),
                tooltip: "Auswahl öffnen",
                onPressed: () {
                  FocusScope.of(context).unfocus();
                  showCustomDialog(
                      context,
                      DatetimePickerDialog(
                        datetime: date,
                        timePicker: true,
                        setDatetime: (date) => setNewDateTime(date),
                      ));
                },
              )
            : Container(),
      ],
    );
  }
}
