import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class LockedNavigatorWidget extends StatefulWidget {
  final Widget child;

  const LockedNavigatorWidget({
    Key key,
    this.child,
  }) : super(key: key);

  @override
  _LockedNavigatorWidgetState createState() => _LockedNavigatorWidgetState();

  static _LockedNavigatorWidgetState _currentState;

  static void animate() {
    if (_currentState != null &&
        _currentState.mounted &&
        _currentState.isLocked) {
      if (_currentState.isLeftNavigatorLocked)
        _currentState._animationController.forward();
      else
        SnackBarController.showIconAndText(Icons.save,
            "Bitte den aktuellen Bearbeitungsvorgang abbrechen oder speichern!");
    }
  }

  static bool isLocked() {
    if (_currentState != null && _currentState.mounted) {
      return _currentState.isLocked;
    }
    return false;
  }

  static void setLocked(bool isLocked, {bool lockLeftNavigator = true}) {
    print("[LockedNavigatorWidget] setLocked: $isLocked - $_currentState");
    if (_currentState != null && _currentState.mounted) {
      _currentState.setLocked(isLocked, lockLeftNavigator: lockLeftNavigator);
    }
  }
}

class LockEntry {
  LockEntry(this.lockLeftNavigator);
  bool lockLeftNavigator;
}

class _LockedNavigatorWidgetState extends State<LockedNavigatorWidget>
    with SingleTickerProviderStateMixin {
  AnimationController _animationController;
  Animation<double> _opacityTween;

  List<LockEntry> lockStack;
  bool get isLocked => lockStack.length != 0;
  bool get isLeftNavigatorLocked =>
      isLocked && lockStack.last.lockLeftNavigator;

  @override
  void initState() {
    super.initState();

    lockStack = <LockEntry>[];

    LockedNavigatorWidget._currentState = this;

    _animationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 300));
    _opacityTween =
        Tween<double>(begin: 0, end: 1).animate(_animationController);
  }

  void setLocked(bool isLocked, {bool lockLeftNavigator = true}) {
    print("[LockedNavigatorWidgetState] setLocked: $isLocked");
    setState(() {
      if (isLocked) {
        lockStack.add(LockEntry(lockLeftNavigator));
      } else {
        if (lockStack.length > 0) lockStack.remove(lockStack.last);
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return isLeftNavigatorLocked
        ? MouseRegion(
            onEnter: (event) => _animationController.forward(),
            onExit: (event) => _animationController.reverse(),
            child: Stack(
              children: [
                Row(
                  children: [
                    Expanded(
                      child: Column(
                        children: [
                          Expanded(child: Container()),
                        ],
                      ),
                    ),
                  ],
                ),
                widget.child,
                AnimatedBuilder(
                    animation: _opacityTween,
                    builder: (context, _) {
                      return Container(
                        width: _opacityTween.value == 0
                            ? 0
                            : MediaQuery.of(context).size.width,
                        height: _opacityTween.value == 0
                            ? 0
                            : MediaQuery.of(context).size.height,
                        color: CleanAndroidDesignDefaultColors.background
                            .withOpacity(0.7 * _opacityTween.value),
                        child: Center(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(Icons.save,
                                  size: 33,
                                  color: Colors.grey[600]
                                      .withOpacity(_opacityTween.value)),
                              Container(height: 10),
                              Text("Bitte den aktuellen Bearbeitungsvorgang",
                                  style: TextStyles.defaultTextStyle().copyWith(
                                      color: Colors.grey[600]
                                          .withOpacity(_opacityTween.value))),
                              Text("abbrechen oder speichern!",
                                  style: TextStyles.defaultTextStyle().copyWith(
                                      color: Colors.grey[600]
                                          .withOpacity(_opacityTween.value))),
                              Container(height: 30),
                            ],
                          ),
                        ),
                      );
                    }),
              ],
            ),
          )
        : widget.child;
  }
}
