import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/utils/force_update_key.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class SplitNavigator {
  static ForceUpdateKey updateKey = ForceUpdateKey();
  static bool rightIsShown = false;
  static bool draggableIsShown = false;

  static BuildContext mainContext;

  static NavigatorWidgetState navigatorWidgetState;

  static GlobalKey<NavigatorState> leftNavigatorKey =
      GlobalKey<NavigatorState>();
  static GlobalKey<NavigatorState> rightNavigatorKey =
      GlobalKey<NavigatorState>();
  static GlobalKey<NavigatorState> draggableNavigatorKey =
      GlobalKey<NavigatorState>();

  static bool _singleNavigator;
  static bool singleNavigator(BuildContext context) {
    _singleNavigator ??=
        !kIsWeb || MediaQuery.of(context).size.longestSide < 1200;
    return _singleNavigator;
  }

  static NavigatorState of(SplitNavigatorSide side) {
    if ((_singleNavigator ?? true) ||
        side.number == SplitNavigatorSide.main.number)
      return Navigator.of(mainContext);
    else if (side.number == SplitNavigatorSide.left.number)
      return leftNavigatorKey.currentState;
    else if (side.number == SplitNavigatorSide.right.number)
      return rightNavigatorKey.currentState;
    else if (side.number == SplitNavigatorSide.draggable.number)
      return draggableNavigatorKey.currentState;
    else
      return leftNavigatorKey.currentState;
  }

  static void closeSecondNavigator() {
    if (_singleNavigator != null && !_singleNavigator)
      SplitNavigator.of(SplitNavigatorSide.right)
          .popUntil((route) => route is NoPageRoute);
  }

  static void closeDraggableNavigator() {
    if (_singleNavigator != null && !_singleNavigator)
      SplitNavigator.of(SplitNavigatorSide.draggable)
          .popUntil((route) => route is NoPageRoute);
  }

  static void setDraggablePositionAndSize({
    Offset position,
    double height,
    bool heightAbsoluteValue = true,
    double width,
    bool widthAbsoluteValue = true,
  }) {
    if (_singleNavigator != null && !_singleNavigator)
      navigatorWidgetState.setDraggablePositionAndSize(
        position: position,
        height: height,
        heightAbsoluteValue: heightAbsoluteValue,
        width: width,
        widthAbsoluteValue: widthAbsoluteValue,
      );
  }
}

class SplitNavigatorSide {
  final int number;
  const SplitNavigatorSide(this.number);

  static final SplitNavigatorSide main = const SplitNavigatorSide(0);
  static final SplitNavigatorSide left = const SplitNavigatorSide(1);
  static final SplitNavigatorSide right = const SplitNavigatorSide(2);
  static final SplitNavigatorSide draggable = const SplitNavigatorSide(3);
}
